/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * 
 * Copyright (C) 1999, Mark Spencer
 *
 * Mark Spencer <markster@linux-support.net>
 *
 * Original Super 'Wait' Application
 * Contributed By Christopher Wade <clwade@sparco.com>
 *
 * refactored and retooled to WaitIVR 
 * by Anthony Minessale <anthmct@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/options.h>
#include <asterisk/musiconhold.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/lock.h>
#include <asterisk/utils.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

static char *app = "WaitIVR";
static char *synopsis = "IVR Helper Application";
static char *tdesc = 
"  WaitIVR(<timeout>[ms]|<count>|[m]<valid_dtmf>[|[[<context>|]<extension>|]<priority>]\n\n"
"Wait for <timeout> seconds (express as milliseconds with trailing 'ms')\n"
"If an extension of length <count> composed of digits listed in <valid_dtmf> is entered\n"
"The application will send the channel to that extension regardless of it's existance.\n"
"The occurance of 'm' anywhere in <valid_dtmf> will play music whilst waiting.\n\n"
"<timeout>, <valid_dtmf> and <count> are required but <context>, <exten> and <priority> are optional\n"
"values to override the location of where to go if a timeout is encountered [ala Goto()]\n\n"
"e.g.\n\n"
"Wait 5 seconds for a 1 digit extension either 1 or 2 \n"
"WaitIVR(5|1|12)\n\n"
"Wait 5 seconds while playing MOH for a 1 digit extension either 1 or 2 \n"
"WaitIVR(5|1|m12)\n\n"
"Wait 10 seconds for a 2 digit extension containing any digits in [1234567890] \n"
" - WaitIVR(10|2|1234567890)\n\n"
"Wait 1.5 seconds for a 2 digit extension containing any digits in [1234567890] \n"
" - WaitIVR(1500ms|3|1234567890)\n\n"
"Wait 1.5 seconds for a 2 digit extension containing any digits in [1234567890] \n"
"if no extension in entered then goto exten 'myto,1' in the current context.\n"
" - WaitIVR(1500ms|2|1234567890|${CONTEXT}|myto|1)\n\n";



STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static char * ast_next_opt(char *optstr, char delim) {
	char *newstr = NULL;
	
	if((newstr = strchr(optstr, delim))) {
		*newstr = '\0';
		newstr++;
	}

	return newstr;
}


static int swait_exec(struct ast_channel *chan, void *data)
{
	struct localuser *u;
	int ms = 0, res = 0, i = 0;
	char *wait = NULL, *options = NULL, *fail_to = NULL, *isms = NULL, *inbuf = NULL, *len_a = NULL;
	double delay = 0;
	struct ast_frame *f = NULL;
	struct timeval started, now;
    long elapsed = 0;
	size_t len = 1;
	
	LOCAL_USER_ADD(u);
	gettimeofday(&started, NULL);
	gettimeofday(&now, NULL);

	if ((wait = ast_strdupa(data))) {
		if((len_a = ast_next_opt(wait,'|'))) {
			if((options = ast_next_opt(len_a,'|')))
				fail_to = ast_next_opt(options,'|');
		}
	}
	
	if(wait && !ast_strlen_zero(wait) && options && len_a) {
		if((isms = strstr(wait, "ms"))) {
			*isms = '\0';
			isms++;
		}
		if((delay = atof(wait)))
			ms = delay * (isms ? 1 : 1000);
		else if (chan->pbx)
			ms = chan->pbx->rtimeout * 1000;
		else
			ms = 10000;
		
		if (strchr(options, 'm') && ast_moh_start(chan, NULL))
			ast_log(LOG_WARNING, "Unable to start music on hold on channel %s\n", chan->name);

		res = i = 0;
		len = (len_a ? atoi(len_a) : 1) + 1;
		inbuf = alloca(len);
		while(elapsed < ms) {
			gettimeofday(&now, NULL);
			elapsed = (((now.tv_sec * 1000) + now.tv_usec / 1000) - ((started.tv_sec * 1000) + started.tv_usec / 1000));
			if((f = ast_read(chan))) {
				if(f->frametype == AST_FRAME_DTMF) {
					if(strchr(options, (char) f->subclass)) {
						inbuf[i++] = f->subclass;
						res++;
						if(i >= (len - 1))
							elapsed = ms;
					}
				}
				ast_frfree(f);
				f = NULL;
			} else 
				res = -1;
			
		}
		inbuf[i] = '\0';
		
		if (chan->generatordata)
			ast_moh_stop(chan);
		
		if(!res)
			res = ast_parseable_goto(chan, fail_to ? fail_to : "|t|1");
		else if(i) {
			res = ast_explicit_goto(chan, NULL, inbuf, 1);
		}
		
	} else {
		ast_log(LOG_ERROR, "Usage: WaitIVR(<timeout>[ms]|<count>|[m]<valid_dtmf>[|[[context|]extension|]priority]\n");
		res = -1;
	}

	LOCAL_USER_REMOVE(u);
	return res;
		
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, swait_exec, synopsis, tdesc);
}

char *description(void)
{
	return app;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}

